@@ -169,6 +169,14 @@ static void cb_save_toggled (GtkToggleButton *tb, ScreenshotData  *sd)
 
 
 
+/* Set the show_in_folder when the button is toggled */
+static void cb_show_in_folder_toggled (GtkToggleButton *tb, ScreenshotData *sd)
+{
+  sd->show_in_folder = gtk_toggle_button_get_active (tb);
+}
+
+
+
 /* Set the widget active if the toggle button is active */
 static void
 cb_toggle_set_sensi (GtkToggleButton *tb, GtkWidget *widget)
@@ -469,6 +477,8 @@ static gchar
     type = "jpeg";
   else if (G_UNLIKELY (g_str_has_suffix (save_path, ".bmp")))
     type = "bmp";
+  else if (G_UNLIKELY (g_str_has_suffix (save_path, ".webp")))
+    type = "webp";
 
   if (G_UNLIKELY (!gdk_pixbuf_save (screenshot, save_path, type, &error, NULL)))
     {
@@ -630,7 +640,8 @@ cb_dialog_response (GtkWidget *dialog, gint response, ScreenshotData *sd)
   else
     {
       gtk_widget_destroy (dialog);
-      gtk_main_quit ();
+      if (!sd->plugin)
+        gtk_main_quit ();
     }
 }
 
@@ -888,7 +899,7 @@ GtkWidget *screenshooter_region_dialog_new (ScreenshotData *sd, gboolean plugin)
 
 GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
 {
-  GtkWidget *dlg, *grid, *box, *evbox, *label, *radio, *popover;
+  GtkWidget *dlg, *grid, *box, *evbox, *label, *radio, *checkbox, *popover;
   GtkWidget *actions_grid;
 
   GtkListStore *liststore;
@@ -968,6 +979,17 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
   gtk_widget_set_tooltip_text (radio, _("Save the screenshot to a PNG file"));
   gtk_grid_attach (GTK_GRID (actions_grid), radio, 0, 0, 1, 1);
 
+  /* Show in folder checkbox */
+  checkbox = gtk_check_button_new_with_label (_("Show in Folder"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox), sd->show_in_folder);
+  gtk_widget_set_margin_start (checkbox, 25);
+  g_signal_connect (G_OBJECT (checkbox), "toggled",
+                    G_CALLBACK (cb_show_in_folder_toggled), sd);
+  g_signal_connect (G_OBJECT (radio), "toggled",
+                    G_CALLBACK (cb_toggle_set_sensi), checkbox);
+  gtk_widget_set_tooltip_text (checkbox, _("Shows the saved file in the folder"));
+  gtk_grid_attach (GTK_GRID (actions_grid), checkbox, 0, 1, 1, 1);
+
   if (sd->plugin ||
       gdk_display_supports_clipboard_persistence (gdk_display_get_default ()))
     {
@@ -984,7 +1006,7 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
                         G_CALLBACK (cb_clipboard_toggled), sd);
       g_signal_connect (G_OBJECT (radio), "activate",
                         G_CALLBACK (cb_radiobutton_activate), dlg);
-      gtk_grid_attach (GTK_GRID (actions_grid), radio, 0, 1, 1, 1);
+      gtk_grid_attach (GTK_GRID (actions_grid), radio, 0, 2, 1, 1);
     }
 
   /* Open with radio button */
@@ -999,7 +1021,7 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
                     G_CALLBACK (cb_radiobutton_activate), dlg);
   gtk_widget_set_tooltip_text (radio,
                                _("Open the screenshot with the chosen application"));
-  gtk_grid_attach (GTK_GRID (actions_grid), radio, 0, 2, 1, 1);
+  gtk_grid_attach (GTK_GRID (actions_grid), radio, 0, 3, 1, 1);
 
   /* Open with combobox */
   liststore = gtk_list_store_new (4, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_APP_INFO);
@@ -1013,7 +1035,7 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
                                   "pixbuf", 0, NULL);
   populate_liststore (liststore);
   set_default_item (combobox, sd);
-  gtk_grid_attach (GTK_GRID (actions_grid), combobox, 1, 2, 1, 1);
+  gtk_grid_attach (GTK_GRID (actions_grid), combobox, 1, 3, 1, 1);
 
   g_signal_connect (G_OBJECT (combobox), "changed",
                     G_CALLBACK (cb_combo_active_item_changed), sd);
@@ -1030,7 +1052,7 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
       GtkWidget *image;
 
       box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
-      gtk_grid_attach (GTK_GRID (actions_grid), box, 0, 4, 1, 1);
+      gtk_grid_attach (GTK_GRID (actions_grid), box, 0, 5, 1, 1);
 
       radio =
         gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (radio),
@@ -1163,6 +1185,10 @@ gchar
     gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (combobox), "png", _("PNG File"));
     gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (combobox), "jpg", _("JPG File"));
     gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (combobox), "bmp", _("BMP File"));
+
+    if (screenshooter_is_format_supported ("webp"))
+      gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (combobox), "webp", _("WebP File"));
+
     gtk_combo_box_set_active_id (GTK_COMBO_BOX (combobox), extension);
     g_signal_connect (combobox, "changed", G_CALLBACK (cb_combo_file_extension_changed), chooser);
     gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (chooser), combobox);
