@@ -142,6 +142,7 @@ screenshooter_read_rc_file (const gchar *file, ScreenshotData *sd)
   gint show_mouse = 1;
   gint show_border = 1;
   gboolean timestamp = TRUE;
+  gboolean show_in_folder = FALSE;
   gchar *screenshot_dir = g_strdup (default_uri);
   gchar *title = g_strdup (_("Screenshot"));
   gchar *app = g_strdup ("none");
@@ -166,6 +167,7 @@ screenshooter_read_rc_file (const gchar *file, ScreenshotData *sd)
           show_border = xfce_rc_read_int_entry (rc, "show_border", 1);
           timestamp = xfce_rc_read_bool_entry (rc, "timestamp", TRUE);
           enable_imgur_upload = xfce_rc_read_bool_entry (rc, "enable_imgur_upload", TRUE);
+          show_in_folder = xfce_rc_read_bool_entry (rc, "show_in_folder", FALSE);
 
           g_free (app);
           app = g_strdup (xfce_rc_read_entry (rc, "app", "none"));
@@ -206,6 +208,7 @@ screenshooter_read_rc_file (const gchar *file, ScreenshotData *sd)
   sd->last_user = last_user;
   sd->last_extension = last_extension;
   sd->enable_imgur_upload = enable_imgur_upload;
+  sd->show_in_folder = show_in_folder;
 }
 
 
@@ -234,6 +237,7 @@ screenshooter_write_rc_file (const gchar *file, ScreenshotData *sd)
   xfce_rc_write_entry (rc, "last_extension", sd->last_extension);
   xfce_rc_write_entry (rc, "screenshot_dir", sd->screenshot_dir);
   xfce_rc_write_bool_entry (rc, "enable_imgur_upload", sd->enable_imgur_upload);
+  xfce_rc_write_bool_entry (rc, "show_in_folder", sd->show_in_folder);
 
   /* do not save if action was specified from cli */
   if (!sd->action_specified)
@@ -624,3 +628,65 @@ screenshooter_get_gtk_frame_extents (GdkWindow *window,
   return TRUE;
 #endif
 }
+
+
+
+void
+screenshooter_show_file_in_folder (const gchar *save_location)
+{
+  GDBusProxy *proxy;
+  GVariantBuilder *builder;
+  gchar *uri, *startup_id;
+
+  if (save_location == NULL)
+    return;
+
+  uri = g_filename_to_uri (save_location, NULL, NULL);
+  startup_id = g_strdup_printf ("%s-%ld", "xfce4-screenshooter",
+                                g_get_monotonic_time () / G_TIME_SPAN_SECOND);
+  proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                         G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
+                                         NULL,
+                                         "org.freedesktop.FileManager1",
+                                         "/org/freedesktop/FileManager1",
+                                         "org.freedesktop.FileManager1",
+                                         NULL, NULL);
+  builder = g_variant_builder_new (G_VARIANT_TYPE ("as"));
+  g_variant_builder_add (builder, "s", uri);
+
+  g_dbus_proxy_call_sync (proxy, "ShowItems",
+                          g_variant_new ("(ass)", builder, startup_id),
+                          G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL);
+
+  g_variant_builder_unref (builder);
+  g_free (startup_id);
+  g_free (uri);
+}
+
+
+
+gboolean
+screenshooter_is_format_supported (const gchar *format)
+{
+  gboolean result = FALSE;
+  GSList *supported_formats;
+  gchar *name;
+
+  supported_formats = gdk_pixbuf_get_formats ();
+
+  for (GSList *lp = supported_formats; lp != NULL; lp = lp->next)
+    {
+      name = gdk_pixbuf_format_get_name (lp->data);
+      if (G_UNLIKELY (g_strcmp0 (name, format) == 0))
+        {
+          result = TRUE;
+          g_free (name);
+          break;
+        }
+      g_free (name);
+    }
+
+  g_slist_free_1 (supported_formats);
+
+  return result;
+}
